#include "TCPClientSocket.h"

TCPClientSocket::TCPClientSocket(const char *Ip, unsigned short Port)
: TCPBaseSocket()
{
	if (m_Socket == INVALID_SOCKET)
		return;

	LPHOSTENT lpHostEntry = gethostbyname(Ip);

	if (lpHostEntry == NULL)
	{
		Close();
		return;
	}

	m_AddrIn.sin_family = AF_INET;
	m_AddrIn.sin_addr = *((LPIN_ADDR)*lpHostEntry->h_addr_list);
	m_AddrIn.sin_port = htons(Port);

	if (connect(m_Socket, (LPSOCKADDR)&m_AddrIn, sizeof(struct sockaddr)) == SOCKET_ERROR)
	{
		Close();
		return;
	}
}

TCPClientSocket::TCPClientSocket(SOCKET &Socket, SOCKADDR_IN &AddrIn)
: TCPBaseSocket(Socket, AddrIn)
{}

TCPClientSocket::~TCPClientSocket()
{}

void TCPClientSocket::SetBlocking(bool Block)
{
	u_long IBlock = 0;

	if (!Block)
		IBlock = 1;
			
	ioctlsocket(m_Socket, FIONBIO, &IBlock);
}

TCPSocketErrorLevel TCPClientSocket::Read(char *Buffer, unsigned int BufferLen, int &ReceivedBytes)
{
	ReceivedBytes = recv(m_Socket, Buffer, BufferLen, 0);

	if (ReceivedBytes == 0)
		return SEL_GRACEFULLY_CLOSED;

	if (ReceivedBytes < 0)
	{
		switch (WSAGetLastError())
		{
		case WSANOTINITIALISED: /* WSAStartup() first */
			return SEL_NOT_INITIALIZED;
		case WSAENETDOWN: /* Network subsystem failed */
			return SEL_NET_DOWN;
		case WSAEFAULT: /* The buf parameter is not completely contained in a valid part of the user address space */
			return SEL_BUF_FAULT;
		case WSAENOTCONN: /* Socket is not connected */
			return SEL_NOT_CONNECTED;
		case WSAEINTR: /* Blocking call was canceled using WSACancelBlockingCall() */
			return SEL_BLOCK_CANCELED;
		case WSAEINPROGRESS: /* Another blocking call is already in progress WINSOCK 1.1 */
			return SEL_BLOCK_IN_PROGRESS;
		case WSAENETRESET: /* Connection dropped */
			return SEL_UNEXPECTED_CLOSED;
		case WSAENOTSOCK: /* Should never happen */
			return SEL_UNKNOWN_ERROR;
		case WSAEOPNOTSUPP: /* Should never happen */
			return SEL_UNKNOWN_ERROR;
		case WSAESHUTDOWN: /* Socket was shutdown */
			return SEL_SHUTDOWN;
		case WSAEWOULDBLOCK:
			return SEL_WOULD_BLOCK;
		case WSAEMSGSIZE: /* Message was to big for given buffer ans is truncated */
			return SEL_BUF_TRUNCATED;
		case WSAEINVAL: /* Should never happen */
			return SEL_UNKNOWN_ERROR;
		case WSAECONNABORTED: /* Connection dropped */
			return SEL_UNEXPECTED_CLOSED;
		case WSAETIMEDOUT: /* Connection dropped */
			return SEL_UNEXPECTED_CLOSED;
		case WSAECONNRESET: /* Connection dropped */
			return SEL_UNEXPECTED_CLOSED;
		default:
			return SEL_UNKNOWN_ERROR;
		}
	}

	return SEL_FINE;
}

TCPSocketErrorLevel TCPClientSocket::Write(const char *Buffer, unsigned int BufferLen, int &WrittenBytes)
{
	WrittenBytes = send(m_Socket, Buffer, BufferLen, 0);

	if (WrittenBytes == SOCKET_ERROR)
	{
		/* TODO: More codes */
		switch (WSAGetLastError())
		{
		case WSAEWOULDBLOCK:
			return SEL_WOULD_BLOCK;
		default:
			return SEL_UNKNOWN_ERROR;
		}
	}

	return SEL_FINE;
} 

TCPSocketErrorLevel TCPClientSocket::Write(char *Buffer, int &WrittenBytes)
{
	return Write(Buffer, (int)(strlen(Buffer)+1), WrittenBytes);
}